package com.chelvc.cloud.admin.controller;

import java.util.List;
import javax.validation.Valid;
import javax.validation.constraints.Min;

import com.chelvc.cloud.vehicle.client.HelpCategoryClient;
import com.chelvc.cloud.vehicle.client.dto.HelpCategoryDTO;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryModifyParam;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryPagingParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.oauth.annotation.Authorize;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 问题分类接口
 *
 * @author liude
 * @date 2024/3/4
 */
@Authorize
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class HelpCategoryController {
    private final HelpCategoryClient helpCategoryClient;

    /**
     * 新增问题分类配置
     *
     * @param param 新增参数
     * @return 问题分类配置主键
     */
    @PostMapping("/helpCategory")
    public void addHelpCategory(@RequestBody @Valid HelpCategoryModifyParam param) {
        this.helpCategoryClient.insertHelpCategory(param);
    }

    /**
     * 修改问题分类配置
     *
     * @param id    问题分类配置主键
     * @param param 修改参数
     */
    @PutMapping("/helpCategory/{id}")
    public void updateHelpCategory(@PathVariable("id") @Min(value = 1, message = "问题分类配置主键不能小于1") Long id,
                                   @RequestBody @Valid HelpCategoryModifyParam param) {
        this.helpCategoryClient.updateHelpCategory(id, param);
    }

    /**
     * 查询问题分类配置分页
     *
     * @param param 查询参数
     * @return 问题分类配置分页信息
     */
    @GetMapping("/helpCategory/paging")
    public Pagination<HelpCategoryDTO> getHelpCategoryPaging(@Valid HelpCategoryPagingParam param) {
        return this.helpCategoryClient.getHelpCategoryPaging(param);
    }


    /**
     * 删除问题分类配置
     *
     * @param id 问题分类配置主键
     */
    @PutMapping("/helpCategory/delete/{id}")
    public void deleteHelpCategory(@PathVariable("id") @Min(value = 1, message = "问题分类配置主键不能小于1") Long id) {
        this.helpCategoryClient.deleteHelpCategoryById(id);
    }


    /**
     * 查询问题分类配置列表
     *
     * @return 问题分类配置列表信息
     */
    @GetMapping("/helpCategory/list")
    public List<HelpCategoryDTO> getHelpCategories() {
        return this.helpCategoryClient.selectHelpCategories();
    }
}
