package com.chelvc.cloud.admin.controller;

import java.util.List;
import java.util.Map;
import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;

import com.chelvc.cloud.vehicle.client.GoodsClient;
import com.chelvc.cloud.vehicle.client.MerchantClient;
import com.chelvc.cloud.vehicle.client.constant.GoodsStatus;
import com.chelvc.cloud.vehicle.client.constant.MerchantStatus;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantRankDTO;
import com.chelvc.cloud.vehicle.client.param.MerchantModifyParam;
import com.chelvc.cloud.vehicle.client.param.MerchantPagingParam;
import com.chelvc.cloud.vehicle.client.param.MerchantRankParam;
import com.chelvc.cloud.vehicle.client.param.ReportModifyParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.oauth.annotation.Authorize;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 商家接口
 *
 * @author Woody
 * @date 2023/9/3
 */
@Authorize
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MerchantController {
    private final GoodsClient goodsClient;
    private final MerchantClient merchantClient;

    /**
     * 新增商家
     *
     * @param param 新增参数
     * @return 商家主键
     */
    @PostMapping("/merchant")
    public Long addMerchant(@RequestBody @Valid MerchantModifyParam param) {
        return this.merchantClient.addMerchant(param);
    }

    /**
     * 修改商家
     *
     * @param id    商家主键
     * @param param 修改参数
     */
    @PutMapping("/merchant/{id}")
    public void updateMerchant(@PathVariable("id") @Min(value = 1, message = "商家主键不能小于1") Long id,
                               @RequestBody @Valid MerchantModifyParam param) {
        this.merchantClient.updateMerchant(id, param);
    }

    /**
     * 查询商家分页
     *
     * @param param 查询参数
     * @return 商家分页信息
     */
    @GetMapping("/merchant/paging")
    public Pagination<MerchantDTO> getMerchantPaging(@Valid MerchantPagingParam param) {
        return this.merchantClient.getMerchantPaging(param);
    }

    /**
     * 获取商家信息
     *
     * @param id 商家主键
     * @return 商家信息
     */
    @GetMapping("/merchant/{id}")
    public MerchantDTO getMerchant(@PathVariable("id") @Min(value = 1, message = "商家主键不能小于1") Long id) {
        return null;
    }

    /**
     * 上架商品
     *
     * @param goodsIds 商品ID集合
     */
    @PutMapping("/merchant/list-goods/{goodsIds}")
    public void listGoods(@PathVariable("goodsIds") @NotEmpty(message = "商品ID不能为空") List<Long> goodsIds) {
        this.goodsClient.updateGoodsStatus(goodsIds, GoodsStatus.ONLINE);
    }

    /**
     * 下架商品
     *
     * @param goodsIds 商品ID集合
     */
    @PutMapping("/merchant/delist-goods/{goodsIds}")
    public void delistGoods(@PathVariable("goodsIds") @NotEmpty(message = "商品ID不能为空") List<Long> goodsIds) {
        this.goodsClient.updateGoodsStatus(goodsIds, GoodsStatus.OFFLINE);
    }


    /**
     * 修改商家营业状态
     *
     * @param id             商家主键
     * @param businessStatus 修改参数  营业状态 0-营业中，1-休息中
     */
    @PutMapping("/merchant/updateBusinessStatus/{id}")
    public void updateMerchant(@PathVariable("id") @Min(value = 1, message = "商家主键不能小于1") Long id,
                               String businessStatus) {
        this.merchantClient.updateMerchantBusinessStatus(id, businessStatus);
    }

    /**
     * 根据地区获取商家排名
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchants/rank")
    public List<MerchantRankDTO> listRankMerchants(@Valid MerchantRankParam param) {
        return this.merchantClient.listRankMerchants(param);
    }


    /**
     * 获取商家营业状况
     *
     * @return 获取商家营业状况
     */
    @GetMapping("/merchant/operatConditions")
    public Map<String, Object> listRankMerchants() {
        return this.merchantClient.operatConditions();
    }

    /**
     * 获取经营报表
     *
     * @return 获取经营报表状况
     */
    @PostMapping("/merchant/operatReport")
    public List<Map<String, Object>> operatReport(@RequestBody @Valid ReportModifyParam param) {
        return this.merchantClient.operatReport(param);
    }

    /**
     * 商家审核
     *
     * @param id 商家主键
     * @param
     */
    @GetMapping("/merchantAudit/{id}")
    public void merchantAudit(@PathVariable("id") @Min(value = 1, message = "商家主键不能小于1") Long id,
                              MerchantStatus status) {
        this.merchantClient.merchantAudit(id, status);
    }
}
