package com.chelvc.cloud.admin.controller;

import java.util.HashMap;
import java.util.Map;
import javax.validation.Valid;

import com.chelvc.cloud.admin.copier.UserCopier;
import com.chelvc.cloud.admin.vo.PersonalVO;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.user.client.param.UserPagingParam;
import com.chelvc.cloud.vehicle.client.MerchantClient;
import com.chelvc.cloud.vehicle.client.param.ReportModifyParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.oauth.annotation.Authorize;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 用户接口
 *
 * @author Woody
 * @date 2023/4/30
 */
@Authorize
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserController {
    private final UserClient userClient;
    private final MerchantClient merchantClient;

    /**
     * 获取个人信息
     *
     * @return 用户信息
     */
    @GetMapping("/user/mine")
    public PersonalVO getPersonal() {
        return UserCopier.INSTANCE.copying(this.userClient.getPersonal());
    }

    /**
     * 查询用户分页
     *
     * @param param 查询参数
     * @return 用户分页信息
     */
    @GetMapping("/user/paging")
    public Pagination<UserDTO> getUserPaging(@Valid UserPagingParam param) {
        return this.userClient.getUserPaging(param);
    }

    /**
     * 查询用户注册统计
     *
     * @return 用户统计
     */
    @PostMapping("/user/getRegisterReport")
    public Map<String, Object> getUserRegisterReport(@RequestBody @Valid ReportModifyParam param) {
        Map<String, Object> map = new HashMap<>();
        map.put("user", this.userClient.getUserRegisterReport(param.getYear(), param.getMonth(), param.getType()));
        map.put("merchant", this.merchantClient.getMerchantReport(param));
        return map;
    }


    /**
     * 查询用户注册统计
     *
     * @return 用户统计
     */
    @PostMapping("/user/getRegisterCount")
    public Map<String, Object> getRegisterCount(@RequestBody @Valid ReportModifyParam param) {
        Map<String, Object> map = new HashMap<>();
        Map<String, Object> userRegisterCount = this.userClient.getUserRegisterReportDaily(
                param.getYear(), param.getMonth(), param.getType(), param.getDay()
        );
        Map<String, Object> merchantCount = this.merchantClient.getMerchantReportCount(param);
        map.put("userRegisterCount", ObjectUtils.ifNull(userRegisterCount.get("registerCount"), 0));
        map.put("merchantRegisterCount", ObjectUtils.ifNull(merchantCount.get("registerCount"), 0));
        return map;
    }


}
